/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach.bayernid.proxy.akdb;

import java.io.IOException;

import jakarta.xml.bind.JAXBElement;

import org.springframework.ws.client.core.support.WebServiceGatewaySupport;
import org.springframework.xml.transform.StringResult;
import org.springframework.xml.transform.StringSource;

import akdb.bsp.postkorb.komm.webservice.ObjectFactory;
import akdb.bsp.postkorb.komm.webservice.SendBspNachrichtNative;
import akdb.bsp.postkorb.komm.webservice.SendBspNachrichtNativeOutput;
import de.akdb.egov.bsp.nachrichten.BspNachricht;
import de.akdb.egov.bsp.nachrichten.BspQuittung;
import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.message.BayernIdMessage;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.message.BayernIdMessageMapper;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.message.BayernIdResponse;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
public class BayernIdRemoteService extends WebServiceGatewaySupport {

	private final BayernIdMessageMapper messageMapper;
	private final ObjectFactory objectFactory;

	public BayernIdResponse sendPostfachNachricht(BayernIdMessage message) {
		var sendNachricht = createSendNachricht(message);
		var sendBspNachrichtResponse = sendBspNachricht(sendNachricht);
		var bspQuittung = createBspQuitung(sendBspNachrichtResponse);
		return messageMapper.toBayernIdResponse(bspQuittung);
	}

	SendBspNachrichtNative createSendNachricht(BayernIdMessage message) {
		var bspNachricht = messageMapper.toBspNachricht(message);
		var sendBspNachricht = new SendBspNachrichtNative();
		sendBspNachricht.setBspNachricht(buildBspNachrichtXml(bspNachricht));
		return sendBspNachricht;
	}

	String buildBspNachrichtXml(BspNachricht message) {
		try {
			var result = new StringResult();
			getMarshaller().marshal(message, result);
			return result.toString();
		} catch (IOException e) {
			throw new TechnicalException("Error while marshalling BspNachricht", e);
		}
	}

	SendBspNachrichtNativeOutput sendBspNachricht(SendBspNachrichtNative request) {
		return send(objectFactory.createSendBspNachrichtNative(request)).getValue();
	}

	@SuppressWarnings("unchecked")
	JAXBElement<SendBspNachrichtNativeOutput> send(JAXBElement<SendBspNachrichtNative> jaxbSendBspNachrichtNative) {
		return (JAXBElement<SendBspNachrichtNativeOutput>) getWebServiceTemplate().marshalSendAndReceive(jaxbSendBspNachrichtNative);
	}
	BspQuittung createBspQuitung(SendBspNachrichtNativeOutput bspNachrichtResponse) {
		try {
			return (BspQuittung) getUnmarshaller().unmarshal(new StringSource(bspNachrichtResponse.getBspQuittung()));
		} catch (IOException e) {
			throw new TechnicalException("Error while unmarshalling BspQuittung", e);
		}
	}

}
